from serial import Serial
import serial.tools.list_ports as listports
import time
import tkinter.messagebox as mbox
import tkinter.filedialog as file
import json

class packet_type:
    command = 1
    data_packet = 2
    ping = 3
    error = 4
    returned = 5

class status:
    success = 0
    failure = 1
    connection_error = 3
    usb_error = 2
    unknown = -1

class hubconnection:
    def __init__(self, port: str):
        self.hub = Serial(port,9600)
    
    def send_raw_data(self, data: dict):
        time.sleep(0.24)
        dic = json.dumps(data)
        self.hub.write(bytes(f"�{dic}�\n\r",encoding="utf-8"))
        
    def get_raw_data(self):
        Attempets = 0
        while True:
            if Attempets > 15:
                return None
            time.sleep(0.05)
            if self.hub.in_waiting > 0:
                try:
                    d = json.loads(self.hub.readline().decode("utf-8").split("�")[1])
                    #print(Attempets)
                    #print(d)
                    return d
                except:
                    #print("None")
                    return None
            else:
                print(Attempets)
                Attempets += 1
    
    def send_packet(self,packet):
        packet = {"packet-type":packet_type.data_packet,"packet":packet}
        self.send_raw_data(packet)
        return self.get_raw_data()
    
    def send_ping(self,returndata):
        packet = {"packet-type":packet_type.ping,"return":returndata}
        self.send_raw_data(packet)
        return self.get_raw_data()
    
    def send_command(self,cmd: str,data: list):
        packet = {"packet-type":packet_type.command,"command":cmd,"data":data}
        self.send_raw_data(packet)
        return self.get_raw_data()
    
    def close(self):
        self.hub.close()

def listallports():
    r = listports.comports(include_links=True)
    li = []
    for i in r:
        if i.description == "LEGO Technic Large Hub in FS Mode":
            li.append(i.device)
    return li


from .menu import *
import time
from .textcolors import textcolors
from tkinter.filedialog import *
import os
import sys
from .logging import log

file = None
premake = False

def clear():
    os.system("cls" if os.name == "nt" else "clear")

def run_lmod_maker(args):
    print(
        textcolors.BOLD
        + textcolors.PURPLE
        + "PYToHub is made by @mas6y6 on github\n"
        + textcolors.END
    )
    a = args

    if a[2] == '-file':
        log.info('Asking where to store file')
        d = askdirectory()
        if d == '':
            log.error('No directory to store file')
        else:
            log.info("Creating file")
            f = open(f"{d}/setup.umd",'w+')
            f.write("#Use \"python3 -m pytohub --hmd_maker -h\" to get the help guide to use this file\n")
            f.write("#This file is needed to install your module to your hub\n")
    elif a[2] == '-guide':
        print("""How to use the module_maker

You well be given a file that pytohub will need to use to install the module to your lego hub

    Comments will be ignored #<comment>

    How to use arguments:
    After the command, you need to use a space to implement an argument
    Example: upload_file main.py
    
    File commands:
            make_module : Builds the module folder on the hub and get the hub ready (You can only run this one)
            ARGUMENTS: (<module_name>)

            return_home : Returns to the main directory of the module on the computer and hub
            ARGUMENTS: (None)
            
            upload_file : Uploads a file thats in a directory
            ARGUMENTS: (<file_directory>)
            
            mkdir : Makes a directory in the hub
            ARGUMENTS: (<new_directory>)
            
            chdir : Changes the current directory in the hub
            ARGUMENTS: (<directory>)
            
            finish : Finishs the module when its done (You can only run this one)
            ARGUMENTS: (None)
""")
    else:
        log.error("UNKNOWN ERROR")
    
    #w = sys.argv[0].split('/')
    #w.pop(len(w) - 1)
    #main_mod = '/'.join(w)
    #time.sleep(3)


from .main import run, download_program
from .hmd_maker import run_lmod_maker
from .logging import log
import time
import sys
import os

def show_help():
    print("""
            Pytohub command guide

    Arguements:

        --download : Loads the download menu
        --help : Loads the Pytohub command guide
        --module_maker : Loads the module_maker menu
""")

if __name__ == "__main__":
    program = sys.argv[0]
    if not len(sys.argv) == 1:
        if '--download' in sys.argv:
            download_program()
        
        elif '--module_maker' in sys.argv:
            run_lmod_maker(sys.argv)
            
        elif '--help' in sys.argv:
            show_help()
            
        else:
            log.fatul(f"Unknown arguement {sys.argv[1]} for help use \"--help\"")
    else:
        run()


import os
import time
import sys
import subprocess
import requests
import urllib.request
import tkinter as tk
import tkinter.filedialog as fbox
import tkinter.messagebox as box
from .logging import log
from .textcolors import textcolors
from .legohub import listallports, hubconnection
from .menu import main_menu, second_menu, options_menu
from getkey import getkey, keys
import progressbar
sys.setrecursionlimit(50000)

version = '1.5'
hub_version = 'unknown'
pbar = None
conn = None
processes = {}
ports = []
retry = False
hub = None


def clear():
    os.system("cls" if os.name == "nt" else "clear")


def print_text_logo(version,hub_version):
    print(
        f""" ______   __  __     ______   ______     __  __     __  __     ______    
/\  == \ /\ \_\ \   /\__  _\ /\  __ \   /\ \_\ \   /\ \/\ \   /\  == \   
\ \  _-/ \ \____ \  \/_/\ \/ \ \ \/\ \  \ \  __ \  \ \ \_\ \  \ \  __<   
 \ \_\    \/\_____\    \ \_\  \ \_____\  \ \_\ \_\  \ \_____\  \ \_____\ 
  \/_/     \/_____/     \/_/   \/_____/   \/_/\/_/   \/_____/   \/_____/
  
            (Version: {version}, Hub Version: {hub_version})
                                                                        
""")

def findhub():
    o = 0
    ports = listallports()
    if ports == []:
        log.warning("There is no hubs connected to this computer.")
        print("Start the program on the hub before running PYToHub to list the hub")
        log.fatul("Terminated", exitcode=1)
    elif len(ports) == 1:
        print(
            "There is only one hub connected to this computer do you want to connect it (Y/n)"
        )

    while True:
        clear()
        print("Please select one of your hubs below")
        print("↑↑↑")
        print(ports[o])
        print("↓↓↓")
        print()
        print(f"{o + 1} out of {len(ports)}")
        print()
        print("Navigate using UP/DOWN arrow keys")
        print("Press Q to exit program")
        key = getkey()
        if key == keys.UP:
            if not o == len(ports) - 1:
                o += 1
        elif key == keys.DOWN:
            if not o == 0:
                o -= 1
        elif key == keys.ENTER:
            clear()
            return ports[o]
        elif key == "q":
            exit()
        else:
            pass


def tryconnect(r):
    global hub, retry, hub_version
    log.log("Connecting...")
    for i in range(11):
        if i == 10:
            log.error("Failed to connect to hub (Maybe you disconnected it?)")
            log.fatul("Terminated", exitcode=1)

        try:
            if retry == False:
                hub = hubconnection(r)

            if hub.send_ping(None) == None:
                log.warning("No data. retrying...")
            else:
                log.success("Connected")
                time.sleep(1)
                log.log("Loading hub version")
                hub_version = hub.send_command('version',[])
                if hub_version == None:
                    log.error('Unable to get hub version')
                else:
                    log.log('Received')
                log.log("Loading main menu...")
                break
        except Exception as e:
            log.error(f"An error occurred ({e}). Retrying...")


def upload_file():
    global hub
    log.log("Asking for file to upload...")
    
    w = fbox.askopenfilename(
        filetypes=[("Python Files", "*.py")],
    )
    if w == '':
        log.info('No file selected')
        time.sleep(5)
        return
    
    while True:
        qa = input(f"Do you want to upload {w} (Y/n) :\n")
        if qa == "Y" or qa == "y":
            break
        elif qa == "N" or qa == "n":
            return 
        else:
            pass
    
    filename1 = w.split("/")
    filename = filename1[len(filename1) - 1]
    log.log("Uploading...")
    log.warning("DO NOT DISCONNECT YOUR HUB OR THE FILE WILL BECOME CORRUPTED")
    out = hub.send_command("start_file_upload", [filename])
    time.sleep(1)
    if out["packet-type"] == 4:
        log.error(f"An error occurred: {out['error']}")
        
    widgets = [
        progressbar.Percentage(),
        " ",
        progressbar.ETA(),
        " ",
        textcolors.YELLOW,
        textcolors.BOLD,
        progressbar.Bar(marker="#", left="[", right="]"),
        " ",
        progressbar.Counter(),
        " Lines uploaded",
        textcolors.END
    ]
    f = open(w, "r")
    lines = f.readlines()
    if lines == []:
        log.error("There are no lines in this file upload canceled.")
        time.sleep(3)
        return
    pbar = progressbar.ProgressBar(max_value=len(lines), widgets=widgets)
    pbar.start()
    
    i2 = 0
    for i in lines:
        hub.send_packet(i)
        i2 += 1
        pbar.update(i2)
        time.sleep(0.45)

    log.success("Uploaded successfully")
    log.log("Finishing")
    time.sleep(0.25)
    out = hub.send_command("close_file", [])
    if out["packet-type"] == 4:
        log.error(f"An error occurred: {out['error']}")
        time.sleep(4)
    
    out = hub.send_command("chdir", ['/'])
    if out["packet-type"] == 4:
        log.error(f"An error occurred: {out['error']}")
        time.sleep(4)
    
    out = hub.send_command("chdir", ['modules'])
    if out["packet-type"] == 4:
        log.error(f"An error occurred: {out['error']}")
        time.sleep(4)
    
    out = hub.send_command("stop_upload", [])
    if out["packet-type"] == 4:
        log.error(f"An error occurred: {out['error']}")
        time.sleep(4)
    
    log.success("Finished successfully")
    log.log("Please wait for 3 seconds before returning to main menu")
    time.sleep(1)
    log.log("Please wait for 2 seconds before returning to main menu")
    time.sleep(1)
    log.log("Please wait for 1 seconds before returning to main menu")
    time.sleep(1)

def delete_mods():
    while True:
        mods = hub.send_command('listdir',[])
        mods_files = []
        for i in mods:
            if not len(i.split('.')) == 1:
                mods.append(i)
                
        out = second_menu('Select the mod that you want to remove',mods_files)
        if out[0] == 'Back':
            break
        else:
            out2 = options_menu(f'Are you sure that you want remove \"{out[0]}\"',['Yes','No'],include_exit=False,exitfn=None)
            if out2[1] == 0:
                log.info("Sending command to delete file")
                cmd = hub.send_command('remove_file')
                if cmd['packet-type'] == 4:
                    log.error(f"An error occurred: {cmd['error']}")
                else:
                    log.success("File deleted")
            else:
                pass

def start_main_menu():
    global version, hub_version
    while True:
        out = main_menu(version,hub_version['return'])
        if out[1] == 0:
            log.log("Restarting hub Please wait...")
            hub.send_command("end_conn", [])
            log.success("Command sent")
            log.log(
                "Your hub will restart because if you uploaded data to it changes directorys and glitches the hub"
            )
            exit()
        elif out[1] == 1:
            while True:
                out2 = second_menu(
                "Manage", options=["Upload a python file", "Upload a module"]
                )
                if out2[1] == 0:
                    upload_file()
                elif out2[1] == 1:
                    pass
                elif out2[1] == 2:
                    break
                else:
                    pass
        elif out[1] == 2:
            while True:
                out2 = second_menu(
                    "Manage", options=["View mods", "Delete mods", "Delete files"]
                )
                if out2[1] == 0:
                    delete_mods()
                elif out2[1] == 1:
                    pass
                elif out2[1] == 2:
                    pass
                elif out2[1] == 3:
                    break
                else:
                    pass
        else:
            pass


def show_progress(block_num, block_size, total_size):
    global pbar
    if pbar is None:
        widgets = [
            progressbar.Percentage(),
            " ",
            progressbar.ETA(),
            " ",
            textcolors.YELLOW,
            textcolors.BOLD,
            progressbar.Bar(marker="#", left="[", right="]"),
            textcolors.END,
        ]
        pbar = progressbar.ProgressBar(maxval=total_size, widgets=widgets)
        pbar.start()

    downloaded = block_num * block_size
    if downloaded < total_size:
        pbar.update(downloaded)
    else:
        pbar.finish()
        pbar = None


def download(file, url):
    urllib.request.urlretrieve(url, file, show_progress)

def download_program():
    clear()
    print(
        textcolors.BOLD
        + textcolors.PURPLE
        + "PYToHub is made by @mas6y6 on github\n"
        + textcolors.END
    )
    time.sleep(3)
    print("Loading download menu")
    h = options_menu(
        menuname="Download selection",
        desc="Please pick the lego hub that you are currently using",
        options=[
            "LEGO 45678 Education Spike Prime Hub",
            "LEGO 51515 MINDSTORMS Robot Inventor hub",
        ],
        include_exit=True
    )
    if h[1] == 0:
        log.log("Requesting where to download...")
        dir = fbox.askdirectory()
        if dir == "":
            log.fatul("You did not select a directory...")
        log.log(f"Selected {dir}")
        log.log(f"Downloading... PYToHub.llsp to {dir}")
        try:
            download(
                f"{dir}/PYToHub.llsp",
                "https://github.com/mas6y6/PyToHub-assets/raw/main/PYToHub.llsp",
            )
        except Exception as e:
            log.fatul(
                f"""An unexpeted error has occured 
if you think this is by mistake then report this to (@mas6y6 on github)
{e}"""
            )
        log.success("Downloaded")
        log.log('Make sure that you upload this file to your Spike Prime hub')
    elif h[1] == 1:
        log.log("Requesting where to download...")
        dir = fbox.askdirectory()
        if dir == "":
            log.fatul("You did not select a directory...")
            
        log.log(f"Selected {dir}")
        log.log(f"Downloading... PYToHub.lms to {dir}")
        try:
            download(
                f"{dir}/PYToHub.lms",
                "https://github.com/mas6y6/PyToHub-assets/raw/main/PYTohub.lms",
            )
        except Exception as e:
            log.fatul(
                f"""An unexpeted error has occured 
if you think this is by mistake then report this to (@mas6y6 on github)
{e}"""
            )
        log.success("Downloaded")
        log.log('Make sure that you upload this file to your Mindstorms hub')
    else:
        log.fatul("Unknown error")


def run():
    global hub
    clear()
    print(
        textcolors.BOLD
        + textcolors.PURPLE
        + "PYToHub is made by @mas6y6 on github\n"
        + textcolors.END
    )
    time.sleep(3)
    while True:
        if not hub == None:
            start_main_menu()
        r = findhub()
        tryconnect(r)


from .textcolors import textcolors

class log:
    def fatul(text, exitcode=1):
        print(
            textcolors.RED
            + textcolors.BOLD
            + "[FATUL] "
            + text
            + " ("
            + str(exitcode)
            + ")"
            + textcolors.END
        )
        exit(exitcode)

    def error(text):
        print(textcolors.RED + "[ERROR] " + text + textcolors.END)
    
    def question(text,is_yn=False):
        if is_yn:
            input(textcolors.PURPLE + "[QUESTION] " + textcolors.END + text + " (Y/n) " + " :")
        else:
            input(textcolors.PURPLE + "[QUESTION] " + textcolors.END + text + " :")

    def warning(text):
        print(textcolors.YELLOW + "[WARNING] " + text + textcolors.END)

    def success(text):
        print(textcolors.GREEN + "[SUCCESS] " + text + textcolors.END)

    def successblue(text):
        print(textcolors.BLUE + "[SUCCESS] " + text + textcolors.END)

    def successcyan(text):
        print(textcolors.CYAN + "[SUCCESS] " + text + textcolors.END)

    def log(text):
        print("[INFO] " + text)

    def info(text):
        print("[INFO] " + text)



from getkey import getkey, keys
import os

def clear():
    os.system("cls" if os.name == "nt" else "clear")

def print_text_logo(version,hub_version):
    print(
        f""" ______   __  __     ______   ______     __  __     __  __     ______    
/\  == \ /\ \_\ \   /\__  _\ /\  __ \   /\ \_\ \   /\ \/\ \   /\  == \   
\ \  _-/ \ \____ \  \/_/\ \/ \ \ \/\ \  \ \  __ \  \ \ \_\ \  \ \  __<   
 \ \_\    \/\_____\    \ \_\  \ \_____\  \ \_\ \_\  \ \_____\  \ \_____\ 
  \/_/     \/_____/     \/_/   \/_____/   \/_/\/_/   \/_____/   \/_____/
  
            (Version: {version}, Hub Version: {hub_version})
                                                                        
""")

class textcolors:
    HEADER = "\033[95m"
    BACKGROUND = "\033[107m"
    OKBLUE = "\033[94m"
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    END = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"

def main_menu(g1,g2):
    sel = 0
    
    selects = [
        "Disconnect",
        "Upload",
        "Manage modules"
    ]
    
    while True:
        clear()
        print_text_logo(g1,g2)
        print()
        for i in selects:
            if i == selects[sel]:
                print(textcolors.BACKGROUND+i+textcolors.END)
            else:
                print(i)
        print("\nUse UP/DOWN arrows to navagate")
        key = getkey()
        if key == keys.DOWN:
            if not sel == len(selects) - 1:
                sel += 1
        elif key == keys.UP:
            if not sel == 0:
                sel -= 1
        elif key == keys.ENTER:
            clear()
            return (selects[sel],sel)
        else:
            pass
        
def second_menu(menuname: str,options=[]):
    sel = 0
    options.append("Back")
    while True:
        clear()
        print(menuname)
        print()
        for i in options:
            if i == options[sel]:
                print(textcolors.BACKGROUND+i+textcolors.END)
            else:
                print(i)
        print("\nUse UP/DOWN arrows to navagate")
        key = getkey()
        if key == keys.DOWN:
            if not sel == len(options) - 1:
                sel += 1
        elif key == keys.UP:
            if not sel == 0:
                sel -= 1
        elif key == keys.ENTER:
            clear()
            return (options[sel],sel)
        else:
            pass

def options_menu(menuname: str,options=[],desc=None,include_exit=False, exitfn=exit):
    sel = 0
    while True:
        clear()
        print(menuname)
        print()
        print(desc)
        print()
        for i in options:
            if i == options[sel]:
                print(textcolors.BACKGROUND+i+textcolors.END)
            else:
                print(i)
        if include_exit == True:
            print("\nUse UP/DOWN arrows to navagate or press \"Q\" to exit")
        else:
            print("\nUse UP/DOWN arrows to navagate")
        key = getkey()
        if key == keys.DOWN:
            if not sel == len(options) - 1:
                sel += 1
        elif key == keys.UP:
            if not sel == 0:
                sel -= 1
        elif key == 'q' and include_exit:
            exitfn()
        elif key == keys.ENTER:
            clear()
            return (options[sel],sel)
        else:
            pass



from setuptools import setup

f = open("README.md","r").read()

setup(
    name="pytohub",
    version="v1.0",
    author="mas6y6",
    long_description=f,
    long_description_content_type='text/markdown',
    description="This is a module that can connect to your lego RI hub or lego SPIKE PRIME hub and can directly upload modules to your hub",
    license="MIT",
    packages=["pytohub"],
    install_requires=["get-key","requests","progressbar2","serial"]
)